@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # Author: Mike McKerns (mmckerns @caltech and @uqfoundation)
-# Copyright (c) 2008-2015 California Institute of Technology.
+# Copyright (c) 2008-2016 California Institute of Technology.
 # License: 3-clause BSD.  The full license text is available at:
 #  - http://trac.mystic.cacr.caltech.edu/project/pathos/browser/dill/LICENSE
 """
@@ -56,7 +56,6 @@ import hmac
 import os
 import logging
 import optparse
-import curses
 #import __hello__
 import threading
 import socket
@@ -69,12 +68,17 @@ try:
     HAS_ALL = True
 except ImportError: # Ubuntu
     HAS_ALL = False
+try:
+    #import curses
+    #from curses import textpad, panel
+    HAS_CURSES = True
+except ImportError: # Windows
+    HAS_CURSES = False
 try:
     import ctypes
     HAS_CTYPES = True
 except ImportError: # MacPorts
     HAS_CTYPES = False
-from curses import textpad, panel
 
 # helper objects
 class _class:
@@ -287,16 +291,20 @@ d['MethodDescriptorType'] = type.__dict__['mro']
 d['WrapperDescriptorType'] = type.__repr__
 a['WrapperDescriptorType2'] = type.__dict__['__module__']
 # built-in functions (CH 2)
-if PY3: _methodwrap = (1).__lt__
-else: _methodwrap = (1).__cmp__
+if PY3: 
+    _methodwrap = (1).__lt__
+else: 
+    _methodwrap = (1).__cmp__
 d['MethodWrapperType'] = _methodwrap
 a['StaticMethodType'] = staticmethod(_method)
 a['ClassMethodType'] = classmethod(_method)
 a['PropertyType'] = property()
 d['SuperType'] = super(Exception, _exception)
 # string services (CH 7)
-if PY3: _in = _bytes
-else: _in = _str
+if PY3: 
+    _in = _bytes
+else: 
+    _in = _str
 a['InputType'] = _cstrI = StringIO(_in)
 a['OutputType'] = _cstrO = StringIO()
 # data types (CH 8)
@@ -363,7 +371,7 @@ except NameError:
     # built-in constants (CH 4)
     a['QuitterType'] = quit
     d['ExitType'] = a['QuitterType']
-try: # numpy
+try: # numpy #FIXME: slow... 0.05 to 0.1 sec to import numpy
     from numpy import ufunc as _numpy_ufunc
     from numpy import array as _numpy_array
     from numpy import int32 as _numpy_int32
@@ -468,14 +476,18 @@ x['CSVDictWriterType'] = csv.DictWriter(_cstrO,{})
 x['HashType'] = hashlib.md5()
 x['HMACType'] = hmac.new(_in)
 # generic operating system services (CH 15)
-#x['CursesWindowType'] = _curwin = curses.initscr() #FIXME: messes up tty
-#x['CursesTextPadType'] = textpad.Textbox(_curwin)
-#x['CursesPanelType'] = panel.new_panel(_curwin)
+if HAS_CURSES: pass
+    #x['CursesWindowType'] = _curwin = curses.initscr() #FIXME: messes up tty
+    #x['CursesTextPadType'] = textpad.Textbox(_curwin)
+    #x['CursesPanelType'] = panel.new_panel(_curwin)
 if HAS_CTYPES:
     x['CCharPType'] = ctypes.c_char_p()
     x['CWCharPType'] = ctypes.c_wchar_p()
     x['CVoidPType'] = ctypes.c_void_p()
-    x['CDLLType'] = _cdll = ctypes.CDLL(None)
+    if sys.platform[:3] == 'win':
+        x['CDLLType'] = _cdll = ctypes.cdll.msvcrt
+    else:
+        x['CDLLType'] = _cdll = ctypes.CDLL(None)
     x['PyDLLType'] = _pydll = ctypes.pythonapi
     x['FuncPtrType'] = _cdll._FuncPtr()
     x['CCharArrayType'] = ctypes.create_string_buffer(1)
@@ -524,6 +536,8 @@ else:
 
 # -- cleanup ----------------------------------------------------------------
 a.update(d) # registered also succeed
+if sys.platform[:3] == 'win':
+    os.close(_filedescrip) # required on win32
 os.remove(_tempfile)
 
 
