@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # Author: Mike McKerns (mmckerns @caltech and @uqfoundation)
-# Copyright (c) 2008-2015 California Institute of Technology.
+# Copyright (c) 2008-2016 California Institute of Technology.
 # License: 3-clause BSD.  The full license text is available at:
 #  - http://trac.mystic.cacr.caltech.edu/project/pathos/browser/dill/LICENSE
 """
@@ -11,12 +11,22 @@ Methods for detecting objects leading to pickling failures.
 from __future__ import absolute_import, with_statement
 import dis
 from inspect import ismethod, isfunction, istraceback, isframe, iscode
-from inspect import getmodule
 from .pointers import parent, reference, at, parents, children
 
 from .dill import _trace as trace
 from .dill import PY3
 
+def getmodule(object, _filename=None, force=False):
+    """get the module of the object"""
+    from inspect import getmodule as getmod
+    module = getmod(object, _filename)
+    if module or not force: return module
+    if PY3: builtins = 'builtins'
+    else: builtins = '__builtin__'
+    builtins = __import__(builtins)
+    from .source import getname
+    name = getname(object, force=True)
+    return builtins if name in vars(builtins).keys() else None
 
 def outermost(func): # is analogous to getsource(func,enclosing=True)
     """get outermost enclosing object (i.e. the outer function in a closure)
@@ -163,11 +173,11 @@ def nestedglobals(func, recurse=True):
             names.update(nestedglobals(co, recurse=True))
     return list(names)
 
-def referredglobals(func, recurse=True):
+def referredglobals(func, recurse=True, builtin=False):
     """get the names of objects in the global scope referred to by func"""
-    return globalvars(func, recurse).keys()
+    return globalvars(func, recurse, builtin).keys()
 
-def globalvars(func, recurse=True):
+def globalvars(func, recurse=True, builtin=False):
     """get objects defined in global scope that are referred to by func
 
     return a dict of {name:object}"""
@@ -175,21 +185,50 @@ def globalvars(func, recurse=True):
         im_func = '__func__'
         func_code = '__code__'
         func_globals = '__globals__'
+        func_closure = '__closure__'
     else:
         im_func = 'im_func'
         func_code = 'func_code'
         func_globals = 'func_globals'
+        func_closure = 'func_closure'
     if ismethod(func): func = getattr(func, im_func)
     if isfunction(func):
-        globs = {} #FIXME: vars(getmodule(object)) # get dict of __builtins__
-        globs.update(getattr(func, func_globals) or {})
+        globs = vars(getmodule(sum)) if builtin else {}
+        # get references from within closure
+        orig_func, func = func, set()
+        for obj in getattr(orig_func, func_closure) or {}:
+            _vars = globalvars(obj.cell_contents, recurse, builtin) or {}
+            func.update(_vars) #XXX: (above) be wary of infinte recursion?
+            globs.update(_vars)
+        # get globals
+        globs.update(getattr(orig_func, func_globals) or {})
+        # get names of references
         if not recurse:
-            func = getattr(func, func_code).co_names # get names
+            func.update(getattr(orig_func, func_code).co_names)
         else:
-            func = set(nestedglobals(getattr(func, func_code)))
+            func.update(nestedglobals(getattr(orig_func, func_code)))
             # find globals for all entries of func
             for key in func.copy(): #XXX: unnecessary...?
-                func.update(globalvars(globs.get(key), recurse=True).keys())
+                nested_func = globs.get(key)
+                if nested_func == orig_func:
+                   #func.remove(key) if key in func else None
+                    continue  #XXX: globalvars(func, False)?
+                func.update(globalvars(nested_func, True, builtin))
+    elif iscode(func):
+        globs = vars(getmodule(sum)) if builtin else {}
+       #globs.update(globals())
+        if not recurse:
+            func = func.co_names # get names
+        else:
+            orig_func = func.co_name # to stop infinite recursion
+            func = set(nestedglobals(func))
+            # find globals for all entries of func
+            for key in func.copy(): #XXX: unnecessary...?
+                if key == orig_func:
+                   #func.remove(key) if key in func else None
+                    continue  #XXX: globalvars(func, False)?
+                nested_func = globs.get(key)
+                func.update(globalvars(nested_func, True, builtin))
     else:
         return {}
     #NOTE: if name not in func_globals, then we skip it...
